@@ -28,28 +28,27 @@ import sre_constants
 
 from collections import defaultdict
 
-from gi.repository import Gdk
 from gi.repository import GLib
 from gi.repository import GObject
 from gi.repository import Gtk
 
-from pynicotine import slskmessages
 from pynicotine.config import config
-from pynicotine.geoip.countrycodes import code2name
 from pynicotine.gtkgui.fileproperties import FileProperties
+from pynicotine.gtkgui.utils import connect_key_press_event
 from pynicotine.gtkgui.utils import copy_file_url
-from pynicotine.gtkgui.utils import keyval_to_hardware_keycode
+from pynicotine.gtkgui.utils import copy_text
+from pynicotine.gtkgui.utils import get_key_press_event_args
 from pynicotine.gtkgui.utils import load_ui_elements
-from pynicotine.gtkgui.utils import triggers_context_menu
+from pynicotine.gtkgui.utils import parse_accelerator
 from pynicotine.gtkgui.widgets.filechooser import choose_dir
 from pynicotine.gtkgui.widgets.iconnotebook import IconNotebook
 from pynicotine.gtkgui.widgets.popupmenu import PopupMenu
 from pynicotine.gtkgui.widgets.theme import set_widget_fg_bg_css
 from pynicotine.gtkgui.widgets.treeview import collapse_treeview
+from pynicotine.gtkgui.widgets.treeview import create_grouping_menu
 from pynicotine.gtkgui.widgets.treeview import initialise_columns
 from pynicotine.gtkgui.widgets.treeview import save_columns
 from pynicotine.gtkgui.widgets.treeview import select_user_row_iter
-from pynicotine.gtkgui.widgets.treeview import set_treeview_selected_row
 from pynicotine.gtkgui.widgets.treeview import show_country_tooltip
 from pynicotine.gtkgui.widgets.treeview import show_file_path_tooltip
 from pynicotine.gtkgui.widgets.theme import update_widget_visuals
@@ -69,19 +68,15 @@ class Searches(IconNotebook):
 
         self.searches = {}
         self.usersearches = {}
-        self.users = {}
 
         IconNotebook.__init__(
             self,
             self.frame.images,
-            angle=config.sections["ui"]["labelsearch"],
             tabclosers=config.sections["ui"]["tabclosers"],
             show_hilite_image=config.sections["notifications"]["notification_tab_icons"],
-            reorderable=config.sections["ui"]["tab_reorderable"],
             notebookraw=self.frame.SearchNotebookRaw
         )
 
-        self.popup_enable()
         self.load_config()
 
         self.wish_list = WishList(frame, self)
@@ -115,58 +110,22 @@ class Searches(IconNotebook):
         if not text:
             return
 
-        users = []
-        room = None
-        search_mode = self.frame.SearchMethod.get_active_id()
-        feedback = None
+        mode = self.frame.SearchMethod.get_active_id()
+        room = self.frame.RoomSearchEntry.get_text()
+        user = self.frame.UserSearchEntry.get_text()
 
-        if search_mode == "global":
-            feedback = self.frame.np.pluginhandler.outgoing_global_search_event(text)
+        search_response = self.frame.np.search.do_search(text, mode, room=room, user=user)
 
-            if feedback is not None:
-                text = feedback[0]
-
-        elif search_mode == "rooms":
-            name = self.frame.RoomSearchEntry.get_text()
-            # Space after Joined Rooms is important, so it doesn't conflict
-            # with any possible real room
-            if name != _("Joined Rooms ") and not name.isspace():
-                room = name
-
-            feedback = self.frame.np.pluginhandler.outgoing_room_search_event(room, text)
-
-            if feedback is not None:
-                (room, text) = feedback
-
-        elif search_mode == "buddies":
-            feedback = self.frame.np.pluginhandler.outgoing_buddy_search_event(text)
-
-            if feedback is not None:
-                text = feedback[0]
-
-        elif search_mode == "user":
-            user = self.frame.UserSearchEntry.get_text().strip()
-
-            if user:
-                users = [user]
-            else:
-                return
-
-            feedback = self.frame.np.pluginhandler.outgoing_user_search_event(users, text)
-
-            if feedback is not None:
-                (users, text) = feedback
-
-        else:
-            log.add_warning("Unknown search mode, not using plugin system. Fix me!")
-            feedback = True
-
-        if feedback is None:
+        if not search_response:
             return
 
-        search_id, searchterm_with_excluded, searchterm_without_excluded = self.frame.np.search.do_search(text, search_mode, users, room)
+        search_id, searchterm_with_excluded, searchterm_without_excluded = search_response
+
+        if mode == "user" and user:
+            self.usersearches[search_id] = [user]
+
+        search = self.create_tab(search_id, searchterm_with_excluded, mode, showtab=True)
 
-        search = self.create_tab(search_id, searchterm_with_excluded, search_mode, showtab=True)
         if search["tab"] is not None:
             self.set_current_page(self.page_num(search["tab"].Main))
 
@@ -181,9 +140,6 @@ class Searches(IconNotebook):
 
             self.frame.SearchCombo.append_text(i)
 
-        if search_mode == "user" and users:
-            self.usersearches[search_id] = users
-
     def set_wishlist_interval(self, msg):
         self.wish_list.set_interval(msg)
 
@@ -263,6 +219,7 @@ class Searches(IconNotebook):
 
         label = fulltext[:length]
         self.append_page(tab.Main, label, tab.on_close, fulltext=fulltext)
+        tab.set_label(self.get_tab_label_inner(tab.Main))
 
     def show_search_result(self, msg, username, country):
 
@@ -279,8 +236,9 @@ class Searches(IconNotebook):
 
         counter = len(search["tab"].all_data) + 1
 
-        # No more things to add because we've reached the max_stored_results limit
-        if counter > config.sections["searches"]["max_stored_results"]:
+        # No more things to add because we've reached the result limit
+        if counter > config.sections["searches"]["max_displayed_results"]:
+            self.frame.np.search.remove_allowed_search_id(msg.token)
             return
 
         search["tab"].add_user_results(msg, username, country)
@@ -296,6 +254,8 @@ class Searches(IconNotebook):
                 search["tab"] = None
                 search["ignore"] = True
 
+            self.frame.np.search.remove_allowed_search_id(tab.id)
+
         self.remove_page(tab.Main)
 
     def update_visuals(self):
@@ -307,6 +267,9 @@ class Searches(IconNotebook):
 
         self.wish_list.update_visuals()
 
+    def server_disconnect(self):
+        self.wish_list.server_disconnect()
+
     def save_columns(self):
         """ Save the treeview state of the currently selected tab """
 
@@ -320,34 +283,6 @@ class Searches(IconNotebook):
                 search["tab"].save_columns()
                 break
 
-    def get_search_id(self, child):
-
-        search_id = None
-        n = self.page_num(child)
-        page = self.get_nth_page(n)
-
-        for search, data in self.searches.items():
-
-            if data["tab"] is None:
-                continue
-            if data["tab"].Main is page:
-                search_id = search
-                break
-
-        return search_id
-
-    def on_tab_popup(self, widget, child):
-
-        search_id = self.get_search_id(child)
-
-        if search_id is None:
-            log.add_warning(_("Search ID was none when clicking tab"))
-            return False
-
-        menu = self.searches[search_id]["tab"].tab_menu
-        menu.popup()
-        return True
-
 
 class Search:
 
@@ -358,6 +293,17 @@ class Search:
 
         # Build the window
         load_ui_elements(self, os.path.join(self.frame.gui_dir, "ui", "search.ui"))
+        load_ui_elements(self, os.path.join(self.frame.gui_dir, "ui", "popovers", "searchfilters.ui"))
+
+        self.key_controller = connect_key_press_event(self.ResultsList, self.on_key_press_event)
+        self.ShowSearchHelp.set_popover(self.AboutSearchFiltersPopover)
+
+        if Gtk.get_major_version() == 4:
+            self.ResultGrouping.set_icon_name("view-list-symbolic")
+            self.ShowSearchHelp.set_icon_name("dialog-question-symbolic")
+        else:
+            self.ResultGrouping.set_image(Gtk.Image.new_from_icon_name("view-list-symbolic", Gtk.IconSize.BUTTON))
+            self.ShowSearchHelp.set_image(Gtk.Image.new_from_icon_name("dialog-question-symbolic", Gtk.IconSize.BUTTON))
 
         self.text = text
         self.searchterm_words_include = [p for p in text.lower().split() if not p.startswith('-')]
@@ -418,8 +364,7 @@ class Search:
         self.column_numbers = list(range(self.resultsmodel.get_n_columns()))
         color_col = 18
         self.cols = cols = initialise_columns(
-            "file_search",
-            self.ResultsList,
+            "file_search", self.ResultsList,
             ["id", _("ID"), 50, "text", color_col],
             ["user", _("User"), 200, "text", color_col],
             ["country", _("Country"), 25, "pixbuf", None],
@@ -451,16 +396,11 @@ class Search:
 
         self.update_visuals()
 
-        """ Filters """
-
-        self.ShowFilters.set_active(config.sections["searches"]["filters_visible"])
-        self.populate_filters()
-
         """ Popup """
 
         self.popup_menu_users = PopupMenu(self.frame)
 
-        self.popup_menu = PopupMenu(self.frame)
+        self.popup_menu = PopupMenu(self.frame, self.ResultsList, self.on_popup_menu)
         self.popup_menu.setup(
             ("#" + "selected_files", None),
             ("", None),
@@ -468,7 +408,8 @@ class Search:
             ("#" + _("Download File(s) _To..."), self.on_download_files_to),
             ("#" + _("Download _Folder(s)"), self.on_download_folders),
             ("#" + _("Download F_older(s) To..."), self.on_download_folders_to),
-            ("#" + _("_Browse Folder"), self.on_browse_folder),
+            ("", None),
+            ("#" + _("_Browse Folder(s)"), self.on_browse_folder),
             ("#" + _("File _Properties"), self.on_file_properties),
             ("", None),
             ("#" + _("Copy _File Path"), self.on_copy_file_path),
@@ -483,15 +424,25 @@ class Search:
             ("#" + _("Copy Search Term"), self.on_copy_search_term),
             ("", None),
             ("#" + _("Clear All Results"), self.on_clear),
-            ("#" + _("Close All Tabs"), self.on_close_all_tabs),
+            ("#" + _("Close All Tabs..."), self.on_close_all_tabs),
             ("#" + _("_Close Tab"), self.on_close)
         )
 
         """ Grouping """
 
-        self.ResultGrouping.set_active(config.sections["searches"]["group_searches"])
+        menu = create_grouping_menu(self.frame.MainWindow, config.sections["searches"]["group_searches"], self.on_group)
+        self.ResultGrouping.set_menu_model(menu)
+
         self.ExpandButton.set_active(config.sections["searches"]["expand_searches"])
 
+        """ Filters """
+
+        self.ShowFilters.set_active(config.sections["searches"]["filters_visible"])
+        self.populate_filters()
+
+    def set_label(self, label):
+        self.tab_menu.set_widget(label)
+
     def on_tooltip(self, widget, x, y, keyboard_mode, tooltip):
 
         country_tooltip = show_country_tooltip(widget, x, y, tooltip, 13, strip_prefix="")
@@ -525,7 +476,7 @@ class Search:
             if len(sfilter) > 6:
                 self.FilterTypeEntry.set_text(str(sfilter[6]))
 
-            self.on_refilter(None)
+            self.on_refilter()
 
         for i in ['0', '128', '160', '192', '256', '320']:
             self.FilterBitrate.append_text(i)
@@ -595,36 +546,16 @@ class Search:
             else:
                 combobox.prepend_text(text)
 
-    def add_user_results(self, msg, user, country):
-
-        if user in self.users:
-            return
-
-        self.users.add(user)
-
-        counter = len(self.all_data) + 1
-
-        inqueue = msg.inqueue
-        ulspeed = msg.ulspeed
-        h_speed = human_speed(ulspeed)
-
-        if msg.freeulslots:
-            imdl = "Y"
-            inqueue = 0
-        else:
-            imdl = "N"
-
-        color_id = (imdl == "Y" and "search" or "searchq")
-        color = config.sections["ui"][color_id] or None
-
-        h_queue = humanize(inqueue)
+    def add_result_list(self, result_list, counter, user, country, inqueue, ulspeed, h_speed,
+                        imdl, h_queue, color, private=False):
+        """ Adds a list of search results to the treeview. Lists can either contain publicly or
+        privately shared files. """
 
         update_ui = False
-        maxstoredresults = config.sections["searches"]["max_stored_results"]
+        max_results = config.sections["searches"]["max_displayed_results"]
 
-        for result in msg.list:
-
-            if counter > maxstoredresults:
+        for result in result_list:
+            if counter > max_results:
                 break
 
             fullpath = result[1]
@@ -640,14 +571,28 @@ class Search:
                 log.add_search(_("Filtered out inexact or incorrect search result " + fullpath + " from user " + user))
                 continue
 
-            fullpath_split = reversed(fullpath.split('\\'))
-            name = next(fullpath_split)
+            fullpath_split = fullpath.split('\\')
+
+            if config.sections["ui"]["reverse_file_paths"]:
+                # Reverse file path, file name is the first item. next() retrieves the name and removes
+                # it from the iterator.
+                fullpath_split = reversed(fullpath_split)
+                name = next(fullpath_split)
+
+            else:
+                # Regular file path, file name is the last item. Retrieve it and remove it from the list.
+                name = fullpath_split.pop()
+
+            # Join the resulting items into a folder path
             directory = '\\'.join(fullpath_split)
 
             size = result[2]
             h_size = human_size(size)
             h_bitrate, bitrate, h_length, length = get_result_bitrate_length(size, result[4])
 
+            if private:
+                name = "[PRIVATE FILE]  " + name
+
             is_result_visible = self.append(
                 [
                     GObject.Value(GObject.TYPE_UINT64, counter),
@@ -677,6 +622,42 @@ class Search:
 
             counter += 1
 
+        return update_ui, counter
+
+    def add_user_results(self, msg, user, country):
+
+        if user in self.users:
+            return
+
+        self.users.add(user)
+
+        counter = len(self.all_data) + 1
+
+        inqueue = msg.inqueue
+        ulspeed = msg.ulspeed
+        h_speed = human_speed(ulspeed)
+
+        if msg.freeulslots:
+            imdl = "Y"
+            inqueue = 0
+        else:
+            imdl = "N"
+
+        h_queue = humanize(inqueue)
+
+        color_id = (imdl == "Y" and "search" or "searchq")
+        color = config.sections["ui"][color_id] or None
+
+        update_ui, counter = self.add_result_list(
+            msg.list, counter, user, country, inqueue, ulspeed, h_speed, imdl, h_queue, color)
+
+        if msg.privatelist:
+            update_ui_private, counter = self.add_result_list(
+                msg.privatelist, counter, user, country, inqueue, ulspeed, h_speed, imdl, h_queue, color, private=True)
+
+            if not update_ui and update_ui_private:
+                update_ui = True
+
         if update_ui:
             # If this search wasn't initiated by us (e.g. wishlist), and the results aren't spoofed, show tab
             if not self.showtab:
@@ -694,15 +675,12 @@ class Search:
 
         self.all_data.append(row)
 
-        if self.numvisibleresults >= config.sections["searches"]["max_displayed_results"]:
-            return False
-
         if not self.check_filter(row):
             return False
 
         iterator = self.add_row_to_model(row)
 
-        if self.ResultGrouping.get_active_id() != "ungrouped":
+        if self.grouping_mode != "ungrouped":
             # Group by folder or user
 
             if self.ExpandButton.get_active():
@@ -714,14 +692,15 @@ class Search:
                 if path is not None:
                     self.ResultsList.expand_to_path(path)
             else:
-                collapse_treeview(self.ResultsList, self.ResultGrouping.get_active_id())
+                collapse_treeview(self.ResultsList, self.grouping_mode)
 
         return True
 
     def add_row_to_model(self, row):
-        counter, user, flag, immediatedl, h_speed, h_queue, directory, filename, h_size, h_bitrate, h_length, bitrate, fullpath, country, size, speed, queue, length, color = row
+        (counter, user, flag, immediatedl, h_speed, h_queue, directory, filename, h_size, h_bitrate,
+            h_length, bitrate, fullpath, country, size, speed, queue, length, color) = row
 
-        if self.ResultGrouping.get_active_id() != "ungrouped":
+        if self.grouping_mode != "ungrouped":
             # Group by folder or user
 
             empty_int = 0
@@ -755,7 +734,7 @@ class Search:
 
             parent = self.usersiters[user]
 
-            if self.ResultGrouping.get_active_id() == "folder_grouping":
+            if self.grouping_mode == "folder_grouping":
                 # Group by folder
 
                 if directory not in self.directoryiters:
@@ -803,7 +782,7 @@ class Search:
             types = []
             for i in row:
                 types.append(type(i))
-            log.add_warning(_("Search row error: %(exception)s %(row)s"), {'exception': e, 'row': row})
+            log.add("Search row error: %(exception)s %(row)s", {'exception': e, 'row': row})
             iterator = None
 
         return iterator
@@ -811,7 +790,7 @@ class Search:
     def check_digit(self, sfilter, value, factorize=True):
 
         op = ">="
-        if sfilter[:1] in (">", "<", "="):
+        if sfilter.startswith((">", "<", "=")):
             op, sfilter = sfilter[:1] + "=", sfilter[1:]
 
         if not sfilter:
@@ -929,6 +908,42 @@ class Search:
 
         return True
 
+    def update_filter_counter(self, count):
+
+        if count > 0:
+            self.FilterLabel.set_text(_("Result Filters") + " *")
+        else:
+            self.FilterLabel.set_text(_("Result Filters"))
+
+        self.FilterLabel.set_tooltip_text("%d active filter(s)" % count)
+
+    def update_results_model(self):
+
+        self.ResultsList.set_model(None)
+
+        self.usersiters.clear()
+        self.directoryiters.clear()
+        self.resultsmodel.clear()
+        self.numvisibleresults = 0
+
+        for row in self.all_data:
+            if self.check_filter(row):
+                self.add_row_to_model(row)
+
+        # Update number of visible results
+        self.update_result_counter()
+        self.update_filter_counter(self.active_filter_count)
+
+        self.ResultsList.set_model(self.resultsmodel)
+
+        if self.grouping_mode != "ungrouped":
+            # Group by folder or user
+
+            if self.ExpandButton.get_active():
+                self.ResultsList.expand_all()
+            else:
+                collapse_treeview(self.ResultsList, self.grouping_mode)
+
     def set_filters(self, enable, f_in, f_out, size, bitrate, freeslot, country, f_type):
 
         self.filters = {
@@ -984,21 +999,17 @@ class Search:
         if freeslot:
             self.active_filter_count += 1
 
-        self.usersiters.clear()
-        self.directoryiters.clear()
-        self.resultsmodel.clear()
-        self.numvisibleresults = 0
+        self.update_results_model()
 
-        for row in self.all_data:
-            if self.numvisibleresults >= config.sections["searches"]["max_displayed_results"]:
-                break
+    def add_popup_menu_user(self, popup, user):
 
-            if self.check_filter(row):
-                self.add_row_to_model(row)
+        popup.setup_user_menu(user)
+        popup.setup(
+            ("", None),
+            ("#" + _("Select User's Transfers"), self.on_select_user_results, user)
+        )
 
-        # Update number of visible results
-        self.update_result_counter()
-        self.update_filter_counter(self.active_filter_count)
+        popup.toggle_user_items()
 
     def populate_popup_menu_users(self):
 
@@ -1007,18 +1018,18 @@ class Search:
         if not self.selected_users:
             return
 
-        for user in self.selected_users:
-            popup = PopupMenu(self.frame)
-            popup.setup_user_menu(user)
-            popup.setup(
-                ("", None),
-                ("#" + _("Select User's Transfers"), self.on_select_user_results, user)
-            )
+        # Multiple users, create submenus for each user
+        if len(self.selected_users) > 1:
+            for user in self.selected_users:
+                popup = PopupMenu(self.frame)
+                self.add_popup_menu_user(popup, user)
+                self.popup_menu_users.setup(
+                    (">" + user, popup)
+                )
+            return
 
-            popup.toggle_user_items()
-            self.popup_menu_users.setup(
-                (">" + user, popup)
-            )
+        # Single user, add items directly to "User(s)" submenu
+        self.add_popup_menu_user(self.popup_menu_users, self.selected_users[0])
 
     def on_select_user_results(self, *args):
 
@@ -1083,44 +1094,33 @@ class Search:
     def save_columns(self):
         save_columns("file_search", self.ResultsList.get_columns())
 
-    def on_list_clicked(self, widget, event):
-
-        if triggers_context_menu(event):
-            set_treeview_selected_row(widget, event)
-            return self.on_popup_menu()
-
-        pathinfo = widget.get_path_at_pos(event.x, event.y)
-
-        if pathinfo is None:
-            widget.get_selection().unselect_all()
-
-        elif event.button == 1 and event.type == Gdk.EventType._2BUTTON_PRESS:
-            self.select_results()
-            self.on_download_files()
-            self.ResultsList.get_selection().unselect_all()
-            return True
-
-        return False
-
-    def on_key_press_event(self, widget, event):
+    def on_row_activated(self, treeview, path, column):
 
         self.select_results()
+        self.on_download_files()
 
-        if event.get_state() & Gdk.ModifierType.CONTROL_MASK and \
-                event.hardware_keycode in keyval_to_hardware_keycode(Gdk.KEY_c):
+        treeview.get_selection().unselect_all()
+
+    def on_key_press_event(self, *args):
+
+        keyval, keycode, state = get_key_press_event_args(*args)
+        self.select_results()
+
+        keycodes, mods = parse_accelerator("<Primary>c")
+
+        if state & mods and keycode in keycodes:
             self.on_copy_file_path()
         else:
             # No key match, continue event
             return False
 
-        widget.stop_emission_by_name("key_press_event")
         return True
 
-    def on_popup_menu(self, *args):
+    def on_popup_menu(self, menu, widget):
 
         self.select_results()
 
-        actions = self.popup_menu.get_actions()
+        actions = menu.get_actions()
         users = len(self.selected_users) > 0
         files = len(self.selected_results) > 0
 
@@ -1128,7 +1128,7 @@ class Search:
                   _("Copy _URL")):
             actions[i].set_enabled(False)
 
-        for i in (_("Download _Folder(s)"), _("Download F_older(s) To..."), _("_Browse Folder"),
+        for i in (_("Download _Folder(s)"), _("Download F_older(s) To..."), _("_Browse Folder(s)"),
                   _("Copy _File Path"), _("Copy Folder U_RL")):
             actions[i].set_enabled(files)
 
@@ -1145,28 +1145,25 @@ class Search:
 
                 break
 
-        self.popup_menu.set_num_selected_files(self.selected_files_count)
-
-        self.popup_menu.popup()
-        return True
+        menu.set_num_selected_files(self.selected_files_count)
 
     def on_browse_folder(self, *args):
 
+        requested_users = set()
         requested_folders = set()
 
         for file in self.selected_results:
             user = file[0]
             folder = file[1].rsplit('\\', 1)[0]
 
-            if folder not in requested_folders:
-                self.frame.browse_user(user, folder)
+            if user not in requested_users and folder not in requested_folders:
+                self.frame.np.userbrowse.browse_user(user, folder=folder)
+
+                requested_users.add(user)
                 requested_folders.add(folder)
 
     def on_file_properties(self, *args):
 
-        if not self.frame.np.transfers:
-            return
-
         data = []
         model, paths = self.ResultsList.get_selection().get_selected_rows()
 
@@ -1187,9 +1184,9 @@ class Search:
             bitratestr = model.get_value(iterator, 9)
             length = model.get_value(iterator, 10)
             fn = model.get_value(iterator, 12)
-            directory = fn.rsplit('\\', 1)[0]
+            directory, filename = fn.rsplit('\\', 1)
             cc = model.get_value(iterator, 13)
-            country = "%s / %s" % (cc, code2name(cc))
+            country = "%s / %s" % (cc, self.frame.np.geoip.country_code_to_name(cc))
 
             data.append({
                 "user": user,
@@ -1211,13 +1208,14 @@ class Search:
 
     def on_download_files(self, *args, prefix=""):
 
-        if not self.frame.np.transfers:
+        if not self.frame.np.active_server_conn:
             return
 
         for file in self.selected_results:
             # Make sure the selected result is not a directory
             if not file[1].endswith('\\'):
-                self.frame.np.transfers.get_file(file[0], file[1], prefix, size=file[2], bitrate=file[3], length=file[4], checkduplicate=True)
+                self.frame.np.transfers.get_file(
+                    file[0], file[1], prefix, size=file[2], bitrate=file[3], length=file[4], checkduplicate=True)
 
     def on_download_files_to_selected(self, selected, data):
         self.on_download_files(prefix=selected)
@@ -1233,7 +1231,7 @@ class Search:
 
     def on_download_folders(self, *args, download_location=""):
 
-        if not self.frame.np.transfers:
+        if not self.frame.np.active_server_conn:
             return
 
         if download_location:
@@ -1255,8 +1253,7 @@ class Search:
 
             requested_folders[user][folder] = download_location
 
-            # First queue the visible search results
-            files = []
+            visible_files = []
             for row in self.all_data:
 
                 # Find the wanted directory
@@ -1264,22 +1261,13 @@ class Search:
                     continue
 
                 destination = self.frame.np.transfers.get_folder_destination(user, folder)
-                counter, user, flag, immediatedl, h_speed, h_queue, directory, filename, h_size, h_bitrate, h_length, bitrate, fullpath, country, size, speed, queue, length, color = row
-                files.append((user, fullpath, destination, size.get_uint64(), bitrate.get_uint64(), length.get_uint64()))
+                (counter, user, flag, immediatedl, h_speed, h_queue, directory, filename,
+                    h_size, h_bitrate, h_length, bitrate, fullpath, country, size, speed,
+                    queue, length, color) = row
+                visible_files.append(
+                    (user, fullpath, destination, size.get_uint64(), bitrate.get_uint64(), length.get_uint64()))
 
-            if config.sections["transfers"]["reverseorder"]:
-                files.sort(key=lambda x: x[1], reverse=True)
-
-            for file in files:
-                user, fullpath, destination, size, bitrate, length = file
-
-                self.frame.np.transfers.get_file(
-                    user, fullpath, destination,
-                    size=size, bitrate=bitrate, length=length, checkduplicate=True
-                )
-
-            # Ask for the rest of the files in the folder
-            self.frame.np.send_message_to_peer(user, slskmessages.FolderContentsRequest(None, folder))
+            self.frame.np.search.request_folder_download(user, folder, visible_files)
 
     def on_download_folders_to_selected(self, selected, data):
         self.on_download_folders(download_location=selected)
@@ -1297,52 +1285,59 @@ class Search:
 
         if self.selected_results:
             user, path = self.selected_results[0][:2]
-            self.frame.clipboard.set_text(path, -1)
+            copy_text(path)
 
     def on_copy_url(self, *args):
 
         if self.selected_results:
             user, path = self.selected_results[0][:2]
-            copy_file_url(user, path, self.frame.clipboard)
+            copy_file_url(user, path)
 
     def on_copy_dir_url(self, *args):
 
         if self.selected_results:
             user, path = self.selected_results[0][:2]
-            copy_file_url(user, path.rsplit('\\', 1)[0] + '\\', self.frame.clipboard)
+            copy_file_url(user, path.rsplit('\\', 1)[0] + '\\')
 
-    def on_group(self, widget):
+    def on_search_settings(self, *args):
+        self.frame.on_settings(page='Searches')
 
-        self.on_refilter(widget)
+    def on_group(self, action, state):
 
-        active = widget.get_active()
+        mode = state.get_string()
+        active = mode != "ungrouped"
 
-        self.ResultsList.set_show_expanders(active)
-        config.sections["searches"]["group_searches"] = active
+        config.sections["searches"]["group_searches"] = mode
         self.cols["id"].set_visible(not active)
+        self.ResultsList.set_show_expanders(active)
         self.ExpandButton.set_visible(active)
 
+        self.grouping_mode = mode
+        self.update_results_model()
+
+        action.set_state(state)
+
     def on_toggle_expand_all(self, widget):
 
         active = self.ExpandButton.get_active()
 
         if active:
             self.ResultsList.expand_all()
-            self.expand.set_from_icon_name("go-up-symbolic", Gtk.IconSize.BUTTON)
+            self.expand.set_property("icon-name", "go-up-symbolic")
         else:
-            collapse_treeview(self.ResultsList, self.ResultGrouping.get_active_id())
-            self.expand.set_from_icon_name("go-down-symbolic", Gtk.IconSize.BUTTON)
+            collapse_treeview(self.ResultsList, self.grouping_mode)
+            self.expand.set_property("icon-name", "go-down-symbolic")
 
         config.sections["searches"]["expand_searches"] = active
 
     def on_toggle_filters(self, widget):
 
         visible = widget.get_active()
-        self.FiltersContainer.set_visible(visible)
+        self.FiltersContainer.set_reveal_child(visible)
         config.sections["searches"]["filters_visible"] = visible
 
     def on_copy_search_term(self, *args):
-        self.frame.clipboard.set_text(self.text, -1)
+        copy_text(self.text)
 
     def on_toggle_remember(self, widget):
 
@@ -1365,6 +1360,7 @@ class Search:
 
         if text in history:
             history.remove(text)
+
         elif len(history) >= 5:
             del history[-1]
 
@@ -1372,8 +1368,6 @@ class Search:
         config.write_configuration()
 
         self.add_combo(widget, text)
-        widget.get_child().set_text(text)
-
         return text
 
     def on_refilter(self, *args):
@@ -1389,17 +1383,7 @@ class Search:
         f_country = self.push_history(self.FilterCountry, "filtercc")
         f_type = self.push_history(self.FilterType, "filtertype")
 
-        self.ResultsList.set_model(None)
         self.set_filters(1, f_in, f_out, f_size, f_br, f_free, f_country, f_type)
-        self.ResultsList.set_model(self.resultsmodel)
-
-        if self.ResultGrouping.get_active_id() != "ungrouped":
-            # Group by folder or user
-
-            if self.ExpandButton.get_active():
-                self.ResultsList.expand_all()
-            else:
-                collapse_treeview(self.ResultsList, self.ResultGrouping.get_active_id())
 
     def on_clear_filters(self, *args):
 
@@ -1417,29 +1401,6 @@ class Search:
         self.FilterInEntry.grab_focus()
         self.on_refilter()
 
-    def on_about_filters(self, *args):
-
-        if not hasattr(self, "AboutSearchFiltersPopover"):
-            load_ui_elements(self, os.path.join(self.frame.gui_dir, "ui", "popovers", "searchfilters.ui"))
-            self.AboutSearchFiltersPopover.set_relative_to(self.ShowChatHelp)
-
-        try:
-            self.AboutSearchFiltersPopover.popup()
-
-        except AttributeError:
-            # GTK <3.22 support
-            self.AboutSearchFiltersPopover.set_transitions_enabled(True)
-            self.AboutSearchFiltersPopover.show()
-
-    def update_filter_counter(self, count):
-
-        if count > 0:
-            self.FilterLabel.set_text(_("Result Filters") + " *")
-        else:
-            self.FilterLabel.set_text(_("Result Filters"))
-
-        self.FilterLabel.set_tooltip_text("%d active filter(s)" % count)
-
     def on_clear(self, *args):
 
         self.all_data = []
@@ -1448,6 +1409,9 @@ class Search:
         self.resultsmodel.clear()
         self.numvisibleresults = 0
 
+        # Allow parsing search result messages again
+        self.frame.np.search.add_allowed_search_id(self.id)
+
         # Update number of visible results
         self.update_result_counter()
 
