@@ -23,13 +23,15 @@
 
 import os
 
-from gi.repository import Gdk
 from gi.repository import GLib
 from gi.repository import Gtk
 
 from pynicotine.config import config
 from pynicotine.gtkgui.utils import load_ui_elements
-from pynicotine.gtkgui.widgets.messagedialogs import option_dialog
+from pynicotine.gtkgui.widgets.dialogs import dialog_hide
+from pynicotine.gtkgui.widgets.dialogs import dialog_show
+from pynicotine.gtkgui.widgets.dialogs import option_dialog
+from pynicotine.gtkgui.widgets.dialogs import generic_dialog
 from pynicotine.gtkgui.widgets.theme import update_widget_visuals
 from pynicotine.gtkgui.widgets.treeview import initialise_columns
 from pynicotine.logfacility import log
@@ -47,14 +49,21 @@ class WishList:
         self.wishes = {}
 
         load_ui_elements(self, os.path.join(self.frame.gui_dir, "ui", "dialogs", "wishlist.ui"))
-        self.WishListDialog.set_transient_for(frame.MainWindow)
+
+        self.dialog = generic_dialog(
+            parent=frame.MainWindow,
+            content_box=self.Main,
+            quit_callback=self.hide,
+            title=_("Search Wishlist"),
+            width=600,
+            height=600
+        )
 
         self.store = Gtk.ListStore(str)
 
         self.column_numbers = list(range(self.store.get_n_columns()))
         cols = initialise_columns(
-            None,
-            self.WishlistView,
+            None, self.WishlistView,
             ["wishes", _("Wishes"), -1, "text", None]
         )
 
@@ -88,10 +97,7 @@ class WishList:
         wish = self.AddWishEntry.get_text()
         self.AddWishEntry.set_text("")
 
-        search_id = self.add_wish(wish)
-
-        if search_id:
-            self.frame.np.search.do_wishlist_search(search_id, wish)
+        self.add_wish(wish)
 
     def on_remove_wish(self, *args):
 
@@ -113,7 +119,7 @@ class WishList:
     def on_clear_wishlist(self, *args):
 
         option_dialog(
-            parent=self.WishListDialog,
+            parent=self.dialog,
             title=_('Clear Wishlist?'),
             message=_('Are you sure you wish to clear your wishlist?'),
             callback=self.clear_wishlist_response
@@ -138,8 +144,6 @@ class WishList:
             "ignore": True
         }
 
-        return search_id
-
     def remove_wish(self, wish):
 
         if wish in self.wishes:
@@ -189,7 +193,7 @@ class WishList:
 
         # Wishlists supported by server?
         if self.interval == 0:
-            log.add_warning(_("The server forbid us from doing wishlist searches."))
+            log.add(_("The server forbid us from doing wishlist searches."))
             return False
 
         searches = config.sections["server"]["autosearch"]
@@ -210,8 +214,10 @@ class WishList:
 
         return True
 
-    def conn_close(self):
+    def server_disconnect(self):
+
         self.disconnected = True
+        self.interval = 0
 
         if self.timer is not None:
             GLib.source_remove(self.timer)
@@ -223,8 +229,8 @@ class WishList:
             update_widget_visuals(widget)
 
     def show(self, *args):
-        self.WishListDialog.present_with_time(Gdk.CURRENT_TIME)
+        dialog_show(self.dialog)
 
-    def quit(self, *args):
-        self.WishListDialog.hide()
+    def hide(self, *args):
+        dialog_hide(self.dialog)
         return True
