@@ -24,7 +24,7 @@ from pynicotine.pluginsystem import BasePlugin
 
 class Plugin(BasePlugin):
 
-    __name__ = "Now Playing Sender"
+    __name__ = "MPRIS Now Playing Sender"
     settings = {
         'rooms': ['testroom'],
     }
