@@ -29,7 +29,6 @@ import sys
 import threading
 import time
 
-from itertools import islice
 from random import uniform
 
 from pynicotine.logfacility import log
@@ -233,7 +232,7 @@ class Connection:
 class PeerConnection(Connection):
 
     __slots__ = ("filereq", "filedown", "fileupl", "filereadbytes", "bytestoread", "piercefw",
-                 "lastcallback", "starttime", "sentbytes2", "readbytes2")
+                 "lastcallback")
 
     def __init__(self, conn=None, addr=None, init=None):
         Connection.__init__(self, conn, addr)
@@ -247,10 +246,6 @@ class PeerConnection(Connection):
         self.lastactive = time.time()
         self.lastcallback = time.time()
 
-        self.starttime = None  # Used for upload bandwidth management
-        self.sentbytes2 = 0
-        self.readbytes2 = 0
-
 
 class PeerConnectionInProgress:
     """ As all p2p connect()s are non-blocking, this class is used to
@@ -462,7 +457,7 @@ class SlskProtoThread(threading.Thread):
         self._conns = {}
         self._connsinprogress = {}
         self._uploadlimit = (self._calc_upload_limit_none, 0)
-        self._downloadlimit = (self._calc_download_limit_by_total, 0)
+        self._downloadlimit = (self._calc_download_limit, 0)
         self._ulimits = {}
         self._dlimits = {}
         self.total_uploads = 0
@@ -539,11 +534,11 @@ class SlskProtoThread(threading.Thread):
         self.server_disconnected = True
         self.server_socket = None
 
-        for i in self._conns.copy():
-            self.close_connection(self._conns, i)
+        for connection in self._conns.copy():
+            self.close_connection(self._conns, connection)
 
-        for i in self._connsinprogress.copy():
-            self.close_connection(self._connsinprogress, i)
+        for connection in self._connsinprogress.copy():
+            self.close_connection(self._connsinprogress, connection)
 
         self._queue.clear()
 
@@ -565,69 +560,38 @@ class SlskProtoThread(threading.Thread):
     def _is_download(conn):
         return conn.__class__ is PeerConnection and conn.filedown is not None
 
-    def _calc_transfer_speed(self, i):
-        curtime = time.time()
+    def _calc_upload_limit(self, limit_disabled=False, limit_per_transfer=False):
 
-        if i.starttime is None:
-            i.starttime = curtime
+        limit = self._uploadlimit[1] * 1024.0
+        self.total_uploads = sum(1 for connection in self._conns.values() if self._is_upload(connection))
 
-        elapsed = curtime - i.starttime
-
-        if elapsed == 0:
+        if limit_disabled:
             return 0
 
-        if self._is_upload(i):
-            return i.sentbytes2 / elapsed
+        if not limit_per_transfer and self.total_uploads > 1:
+            limit = limit / self.total_uploads
 
-        return i.readbytes2 / elapsed
+        return int(max(1024, limit))  # 1 KB/s is the minimum upload speed per transfer
 
     def _calc_upload_limit_by_transfer(self):
-        self.total_uploads = sum(1 for j in self._conns.values() if self._is_upload(j))
-
-        return int(self._uploadlimit[1] * 1024.0)
-
-    def _calc_upload_limit_by_total(self):
-        max_limit = self._uploadlimit[1] * 1024.0
-        bandwidth = 0.0
-        self.total_uploads = 1
-
-        """ Skip first upload
-        If we have 2 or more uploads, we start reducing their individual speeds to
-        stay below the total limit """
-
-        uploads = islice((j for j in self._conns.values() if self._is_upload(j)), 1, None)
-        for j in uploads:
-            bandwidth += self._calc_transfer_speed(j)
-            self.total_uploads += 1
-
-        limit = int(max(1024, max_limit - bandwidth))  # 1 KB/s is the minimum upload speed per transfer
-        return limit
+        return self._calc_upload_limit(limit_per_transfer=True)
 
     def _calc_upload_limit_none(self):
-        self.total_uploads = sum(1 for j in self._conns.values() if self._is_upload(j))
-        return 0
+        return self._calc_upload_limit(limit_disabled=True)
 
-    def _calc_download_limit_by_total(self):
-        max_limit = self._downloadlimit[1] * 1024.0
-        bandwidth = 0.0
-        self.total_downloads = 1
+    def _calc_download_limit(self):
 
-        """ Skip first download
-        If we have 2 or more downloads, we start reducing their individual speeds to
-        stay below the total limit """
+        limit = self._downloadlimit[1] * 1024.0
+        self.total_downloads = sum(1 for connection in self._conns.values() if self._is_download(connection))
 
-        downloads = islice((j for j in self._conns.values() if self._is_download(j)), 1, None)
-        for j in downloads:
-            bandwidth += self._calc_transfer_speed(j)
-            self.total_downloads += 1
-
-        if max_limit == 0:
+        if limit == 0:
             # Download limit disabled
-            limit = 0
-        else:
-            limit = int(max(1024, max_limit - bandwidth))  # 1 KB/s is the minimum download speed per transfer
+            return limit
 
-        return limit
+        if self.total_downloads > 1:
+            limit = limit / self.total_downloads
+
+        return int(max(1024, limit))  # 1 KB/s is the minimum download speed per transfer
 
     def _calc_loops_per_second(self):
         """ Calculate number of loops per second. This value is used to split the
@@ -651,18 +615,6 @@ class SlskProtoThread(threading.Thread):
         if limit > 0:
             limits[connection] = limit
 
-    def _reset_counters(self):
-        curtime = time.time()
-
-        for i in self._conns.values():
-            if self._is_upload(i):
-                i.starttime = curtime
-                i.sentbytes2 = 0
-
-            if self._is_download(i):
-                i.starttime = curtime
-                i.sentbytes2 = 0
-
     """ Connections """
 
     def socket_still_active(self, conn):
@@ -1286,22 +1238,20 @@ class SlskProtoThread(threading.Thread):
 
             elif msg_class is UploadFile and msg_obj.conn in self._conns:
                 self._conns[msg_obj.conn].fileupl = msg_obj
-                self._reset_counters()
 
             elif msg_class is SetDownloadLimit:
-                self._downloadlimit = (self._calc_download_limit_by_total, msg_obj.limit)
+                self._downloadlimit = (self._calc_download_limit, msg_obj.limit)
 
             elif msg_class is SetUploadLimit:
                 if msg_obj.uselimit:
                     if msg_obj.limitby:
-                        callback = self._calc_upload_limit_by_total
+                        callback = self._calc_upload_limit
                     else:
                         callback = self._calc_upload_limit_by_transfer
 
                 else:
                     callback = self._calc_upload_limit_none
 
-                self._reset_counters()
                 self._uploadlimit = (callback, msg_obj.limit)
 
     def read_data(self, conn_obj):
@@ -1326,7 +1276,6 @@ class SlskProtoThread(threading.Thread):
         else:
             # Speed Limited Download data (transfers)
             conn_obj.lastreadlength = limit
-            conn_obj.readbytes2 += len(data)
 
         if not data:
             return False
@@ -1362,7 +1311,6 @@ class SlskProtoThread(threading.Thread):
 
         if conn_obj.fileupl is not None and conn_obj.fileupl.offset is not None:
             conn_obj.fileupl.sentbytes += bytes_send
-            conn_obj.sentbytes2 += bytes_send
 
             totalsentbytes = conn_obj.fileupl.offset + conn_obj.fileupl.sentbytes + len(conn_obj.obuf)
 
