@@ -120,6 +120,7 @@ class Transfers:
 
         self.users = users
         self.network_callback = network_callback
+        self.download_queue_timer_count = -1
         self.downloadsview = None
         self.uploadsview = None
 
@@ -485,9 +486,15 @@ class Transfers:
     def get_user_status(self, msg):
         """ We get a status of a user and if he's online, we request a file from him """
 
-        for i in self.downloads.copy():
-            if msg.user == i.user and i.status in ("Queued", "Getting status", "Establishing connection",
-                                                   "User logged off", "Connection closed by peer", "Cannot connect"):
+        download_statuses = ("Queued", "Getting status", "Establishing connection", "Too many files",
+                             "Too many megabytes", "Pending shutdown.", "User logged off", "Connection closed by peer",
+                             "Cannot connect", "Remote file error")
+
+        upload_statuses = ("Getting status", "Establishing connection", "Disallowed extension",
+                           "User logged off", "Cannot connect", "Cancelled")
+
+        for i in reversed(self.downloads.copy()):
+            if msg.user == i.user and (i.status in download_statuses or i.status.startswith("User limit of")):
                 if msg.status <= 0:
                     i.status = "User logged off"
                     self.abort_transfer(i)
@@ -499,9 +506,8 @@ class Transfers:
                     self.get_file(i.user, i.filename, i.path, i)
 
         # We need a copy due to upload auto-clearing modifying the deque during iteration
-        for i in self.uploads.copy():
-            if msg.user == i.user and i.status in ("Getting status", "Establishing connection",
-                                                   "User logged off", "Cannot connect", "Cancelled"):
+        for i in reversed(self.uploads.copy()):
+            if msg.user == i.user and i.status in upload_statuses:
                 if msg.status <= 0:
                     i.status = "User logged off"
                     self.abort_transfer(i)
@@ -648,24 +654,20 @@ class Transfers:
                 )
 
             elif limits and self.queue_limit_reached(user):
-                uploadslimit = self.config.sections["transfers"]["queuelimit"]
-                limitmsg = "User limit of %i megabytes exceeded" % (uploadslimit)
                 self.queue.append(
-                    slskmessages.UploadDenied(conn=msg.conn.conn, file=msg.file, reason=limitmsg)
+                    slskmessages.UploadDenied(conn=msg.conn.conn, file=msg.file, reason="Too many megabytes")
                 )
 
             elif limits and self.file_limit_reached(user):
-                filelimit = self.config.sections["transfers"]["filelimit"]
-                limitmsg = "User limit of %i files exceeded" % (filelimit)
                 self.queue.append(
-                    slskmessages.UploadDenied(conn=msg.conn.conn, file=msg.file, reason=limitmsg)
+                    slskmessages.UploadDenied(conn=msg.conn.conn, file=msg.file, reason="Too many files")
                 )
 
             elif self.core.shares.file_is_shared(user, filename_utf8, real_path):
                 newupload = Transfer(
                     user=user, filename=msg.file,
                     path=os.path.dirname(real_path), status="Queued",
-                    timequeued=time.time(), size=self.get_file_size(real_path), place=len(self.uploads)
+                    timequeued=time.time(), size=self.get_file_size(real_path)
                 )
                 self._append_upload(user, msg.file, newupload)
 
@@ -677,7 +679,7 @@ class Transfers:
 
             else:
                 self.queue.append(
-                    slskmessages.UploadDenied(conn=msg.conn.conn, file=msg.file, reason="File not shared")
+                    slskmessages.UploadDenied(conn=msg.conn.conn, file=msg.file, reason="File not shared.")
                 )
 
         log.add_transfer("QueueUpload request: User %(user)s, %(msg)s", {
@@ -806,7 +808,7 @@ class Transfers:
         real_path = self.core.shares.virtual2real(msg.file)
 
         if not self.core.shares.file_is_shared(user, msg.file, real_path):
-            return slskmessages.TransferResponse(None, 0, reason="File not shared", req=msg.req)
+            return slskmessages.TransferResponse(None, 0, reason="File not shared.", req=msg.req)
 
         # Is that file already in the queue?
         if self.file_is_upload_queued(user, msg.file):
@@ -822,14 +824,10 @@ class Transfers:
                 limits = False
 
         if limits and self.queue_limit_reached(user):
-            uploadslimit = self.config.sections["transfers"]["queuelimit"]
-            return slskmessages.TransferResponse(None, 0, reason="User limit of %i megabytes exceeded" % (uploadslimit),
-                                                 req=msg.req)
+            return slskmessages.TransferResponse(None, 0, reason="Too many megabytes", req=msg.req)
 
         if limits and self.file_limit_reached(user):
-            filelimit = self.config.sections["transfers"]["filelimit"]
-            limitmsg = "User limit of %i files exceeded" % (filelimit)
-            return slskmessages.TransferResponse(None, 0, reason=limitmsg, req=msg.req)
+            return slskmessages.TransferResponse(None, 0, reason="Too many files", req=msg.req)
 
         # All checks passed, user can queue file!
         self.core.pluginhandler.upload_queued_notification(user, msg.file, real_path)
@@ -850,8 +848,7 @@ class Transfers:
             newupload = Transfer(
                 user=user, filename=msg.file,
                 path=os.path.dirname(real_path), status="Queued",
-                timequeued=time.time(), size=self.get_file_size(real_path),
-                place=len(self.uploads)
+                timequeued=time.time(), size=self.get_file_size(real_path)
             )
             self._append_upload(user, msg.file, newupload)
 
@@ -867,7 +864,7 @@ class Transfers:
         transferobj = Transfer(
             user=user, filename=msg.file,
             path=os.path.dirname(real_path), status="Getting status",
-            req=msg.req, size=size, place=len(self.uploads)
+            req=msg.req, size=size
         )
 
         self.transfer_request_times[transferobj] = time.time()
@@ -1157,6 +1154,7 @@ class Transfers:
                 self.queue.append(slskmessages.UploadFile(i.conn, file=file_handle, size=i.size))
                 i.status = "Transferring"
                 i.file = file_handle
+                i.place = 0
 
                 if self.is_privileged(i.user):
                     i.modifier = _("privileged")
@@ -1378,23 +1376,20 @@ class Transfers:
 
             break
 
-    def conn_close(self, conn, user, error):
+    def conn_close(self, conn):
         """ The remote user has closed the connection either because
         he logged off, or because there's a network problem. """
 
         for i in self.downloads:
             if i.conn == conn:
                 self._conn_close(i, "download")
+                return
 
         # We need a copy due to upload auto-clearing modifying the deque during iteration
         for i in self.uploads.copy():
-            if not isinstance(error, ConnectionRefusedError) and i.conn != conn:
-                continue
-
-            if i.user != user:
-                continue
-
-            self._conn_close(i, "upload")
+            if i.conn == conn:
+                self._conn_close(i, "upload")
+                return
 
     def _conn_close(self, i, transfer_type):
 
@@ -1443,6 +1438,7 @@ class Transfers:
         user = msg.conn.init.target_user
         privileged_user = self.is_privileged(user)
         place = 0
+        transfer = None
 
         if self.config.sections["transfers"]["fifoqueue"]:
             for i in reversed(self.uploads):
@@ -1455,6 +1451,7 @@ class Transfers:
 
                 # Stop counting on the matching file
                 if i.user == user and i.filename == msg.file:
+                    transfer = i
                     break
 
         else:
@@ -1479,6 +1476,7 @@ class Transfers:
                     # Stop counting on the matching file
                     if i.filename == msg.file:
                         should_count = False
+                        transfer = i
 
                     continue
 
@@ -1495,7 +1493,17 @@ class Transfers:
                     if not privileged_user or self.is_privileged(i.user):
                         place += 1
 
-        self.queue.append(slskmessages.PlaceInQueue(msg.conn.conn, msg.file, place))
+        if place > 0:
+            self.queue.append(slskmessages.PlaceInQueue(msg.conn.conn, msg.file, place))
+
+        if transfer is None:
+            return
+
+        # Update queue position in our list of uploads
+        transfer.place = place
+
+        if self.uploadsview:
+            self.uploadsview.update(transfer)
 
     def place_in_queue(self, msg):
         """ The server tells us our place in queue for a particular transfer."""
@@ -2017,7 +2025,10 @@ class Transfers:
 
     def _check_download_queue_timer(self):
 
+        self.download_queue_timer_count = -1
+
         while True:
+            self.download_queue_timer_count += 1
             self.network_callback([slskmessages.CheckDownloadQueue()])
 
             if self.core.exit.wait(180):
@@ -2028,19 +2039,43 @@ class Transfers:
     # Also ask for the queue position of downloads.
     def check_download_queue(self):
 
-        statuslist = ("Cannot connect", "Connection closed by peer", "Local file error", "Remote file error")
+        statuslist_failed = ("Cannot connect", "Connection closed by peer", "Local file error", "Remote file error")
+        statuslist_limited = ("Too many files", "Too many megabytes")
+        reset_count = False
+
+        for transfer in reversed(self.downloads):
+            status = transfer.status
+
+            if (self.download_queue_timer_count >= 4
+                    and (status in statuslist_limited or status.startswith("User limit of"))):
+                # Re-queue limited downloads every 12 minutes
+
+                log.add_transfer("Re-queuing file %(filename)s from user %(user)s in download queue", {
+                    "filename": transfer.filename,
+                    "user": transfer.user
+                })
+                reset_count = True
 
-        for transfer in self.downloads:
-            if transfer.status in statuslist:
                 self.abort_transfer(transfer)
                 self.get_file(transfer.user, transfer.filename, transfer.path, transfer)
 
-            elif transfer.status == "Queued":
+            if status in statuslist_failed:
+                # Retry failed downloads every 3 minutes
+
+                self.abort_transfer(transfer)
+                self.get_file(transfer.user, transfer.filename, transfer.path, transfer)
+
+            if status == "Queued":
+                # Request queue position every 3 minutes
+
                 self.core.send_message_to_peer(
                     transfer.user,
                     slskmessages.PlaceInQueueRequest(None, transfer.filename, transfer.legacy_attempt)
                 )
 
+        if reset_count:
+            self.download_queue_timer_count = 0
+
     def get_queued_uploads(self):
 
         # List of transfer instances of users who are not currently transferring
@@ -2055,7 +2090,7 @@ class Transfers:
         queued_users = {}
 
         # Check users
-        for i in self.uploads:
+        for i in reversed(self.uploads):
             # some file is being transferred
             if i.req is not None or i.conn is not None or i.status == "Getting status":
                 if i.user not in uploading_users:
@@ -2065,7 +2100,7 @@ class Transfers:
                 queued_users[i.user] = self.is_privileged(i.user)
 
         # Check queued uploads
-        for i in self.uploads:
+        for i in reversed(self.uploads):
             if i.status == "Queued" and i.user not in uploading_users:
                 if queued_users[i.user]:  # check if user is privileged
                     list_privileged.append(i)
@@ -2154,11 +2189,11 @@ class Transfers:
         """
 
         if ban_message:
-            banmsg = _("Banned (%s)") % ban_message
+            banmsg = "Banned (%s)" % ban_message
         elif self.config.sections["transfers"]["usecustomban"]:
-            banmsg = _("Banned (%s)") % self.config.sections["transfers"]["customban"]
+            banmsg = "Banned (%s)" % self.config.sections["transfers"]["customban"]
         else:
-            banmsg = _("Banned")
+            banmsg = "Banned"
 
         for upload in self.uploads.copy():
             if upload.user != user:
@@ -2219,7 +2254,7 @@ class Transfers:
         transfer.legacy_attempt = False
         transfer.req = None
         transfer.speed = None
-        transfer.place = None
+        transfer.place = 0
         transfer.timeleft = ""
 
         if transfer.conn is not None:
