@@ -33,8 +33,8 @@ from pynicotine.utils import http_request
 
 
 class Router:
-    def __init__(self, ip, port, wan_ip_type, url_scheme, base_url, root_url):
-        self.ip = ip
+    def __init__(self, ip_address, port, wan_ip_type, url_scheme, base_url, root_url):
+        self.ip_address = ip_address
         self.port = port
         self.search_target = wan_ip_type
         self.url_scheme = url_scheme
@@ -47,7 +47,7 @@ class Router:
 
     @classmethod
     def parse_ssdp_response(cls, ssdp_response, sender):
-        response_headers = dict(ssdp_response.headers)
+        response_headers = {k.upper(): v for k, v in ssdp_response.headers}
 
         if 'LOCATION' not in response_headers:
             log.add_debug('The M-SEARCH response from %s:%d did not contain a Location header.', (sender[0], sender[1]))
@@ -58,7 +58,7 @@ class Router:
         urlparts = urlsplit(response_headers['LOCATION'])
 
         return Router(
-            ip=sender[0],
+            ip_address=sender[0],
             port=sender[1],
             wan_ip_type=response_headers['ST'],
             url_scheme=urlparts.scheme,
@@ -72,9 +72,10 @@ class SSDP:
     multicast_port = 1900
     buffer_size = 4096
     response_time_secs = 2
+    sleep_time_secs = 0.01
 
     @classmethod
-    def list(cls, refresh=False):
+    def list(cls, private_ip=None):
         """ list finds all devices responding to an SSDP search """
 
         log.add_debug('UPnP: Discovering... delay=%s seconds', SSDP.response_time_secs)
@@ -84,6 +85,9 @@ class SSDP:
         sock.setsockopt(socket.IPPROTO_IP, socket.IP_MULTICAST_TTL, 2)
         sock.setblocking(False)
 
+        if private_ip:
+            sock.bind((private_ip, 0))
+
         # Create the WANIPConnection:1 and WANIPConnection:2 request objects
         headers = {
             'HOST': "{}:{}".format(SSDP.multicast_host, SSDP.multicast_port),
@@ -116,7 +120,7 @@ class SSDP:
         time_end = time.time() + SSDP.response_time_secs
 
         while time.time() < time_end:
-            _timeout = 1
+            _timeout = 0
             readable, writable, _ = select.select(inputs, outputs, inputs, _timeout)
 
             for _sock in readable:
@@ -161,17 +165,22 @@ class SSDP:
                     wan_igd2_sent = True
 
             # Cooldown
-            time.sleep(0.4)
+            time.sleep(cls.sleep_time_secs)
 
-        for r in routers:
+        for router in list(routers):
             serial_number, control_url, uuid, svc_type = SSDP._get_router_service_description(
-                r.url_scheme, r.base_url, r.root_url
+                router.url_scheme, router.base_url, router.root_url
             )
 
-            r.serial_number = serial_number
-            r.control_url = control_url
-            r.uuid = uuid
-            r.svc_type = svc_type
+            if svc_type is None:
+                # Invalid entry
+                routers.remove(router)
+                continue
+
+            router.serial_number = serial_number
+            router.control_url = control_url
+            router.uuid = uuid
+            router.svc_type = svc_type
 
         sock.close()
         log.add_debug('UPnP: %s device(s) detected', str(len(routers)))
@@ -192,10 +201,16 @@ class SSDP:
 
         from xml.etree import ElementTree
 
-        response = http_request(url_scheme, base_url, root_url)
-
         # Parse the returned XML and find the <URLBase> and <controlURL> elements
-        xml = ElementTree.fromstring(response)
+        try:
+            response = http_request(url_scheme, base_url, root_url, timeout=2)
+            xml = ElementTree.fromstring(response)
+
+        except Exception as error:
+            # Invalid response
+            log.add_debug('UPnP: Invalid router description response from %s://%s%s: %s',
+                          (url_scheme, base_url, root_url, error))
+            return (None, None, None, None)
 
         serial_number = next(
             (x.text for x in xml.findall(".//{urn:schemas-upnp-org:device-1-0}serialNumber")),
@@ -224,8 +239,9 @@ class SSDP:
 
     @classmethod
     def _is_wanip_service(cls, svc_type):
-        return svc_type == "urn:schemas-upnp-org:service:WANIPConnection:1" \
-            or svc_type == "urn:schemas-upnp-org:service:WANIPConnection:2"
+        return svc_type in ("urn:schemas-upnp-org:service:WANIPConnection:1",
+                            "urn:schemas-upnp-org:service:WANPPPConnection:1",
+                            "urn:schemas-upnp-org:service:WANIPConnection:2")
 
 
 class SSDPMessage:
